@@ -10,6 +10,7 @@
 require(shiny)
 require(shinydashboard)
 require(presize)
+require(ggplot2)
 
 icon <- icon("calculator")
 
@@ -22,6 +23,8 @@ ui <- dashboardPage(skin = "red",
                          titleWidth = 500),
          dashboardSidebar(width = 300,
              sidebarMenu(menuItem("Help/Instructions", tabName = "help")),
+             tags$style(HTML(".irs--shiny .irs-grid-text{ color: #999999;}
+                              .irs--shiny .irs-grid-pol { background-color: #999999 }")),
              sliderInput("conflevel", "Confidence level",
                           value = 0.95, min = 0, max = 1),
              sidebarMenu(
@@ -64,7 +67,7 @@ ui <- dashboardPage(skin = "red",
                  br(),
                  "presize was developed by members of the Swiss Clinical Trials Organisation",
                  br(),
-                 img(src = "scto_ctu_member_cmyk.jpg"),
+                 img(src = "SCTO_Platform_Logo_2020_RZ_SM.jpg"),
                  width = 12
              )
          ),
@@ -112,6 +115,14 @@ server <- function(input, output, session) {
                      c("mu", "sd", "n", "lwr", "upr", "conf.width", "conf.level"),]
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$mean_resetable_input <- renderUI({
+        times <- input$mean_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("mean_n", "Number of observations",
+                         value = NULL, min = 0, step = 1),
+            numericInput("mean_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, step = .1))
+    })
 
     # proportion ----
     output$prop_code <- renderPrint(prop_fn(input, TRUE))
@@ -122,6 +133,14 @@ server <- function(input, output, session) {
                      c("p", "padj", "n", "lwr", "upr", "conf.width", "conf.level"),]
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$prop_resetable_input <- renderUI({
+        times <- input$prop_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("prop_n", "Number of observations",
+                         value = NULL, min = 0, step = 1),
+            numericInput("prop_ciwidth", "Confidence interval width",
+                         value = NULL, step = .1))
+    })
 
     # rate ----
     output$rate_out <- renderPrint(rate_fn(input))
@@ -133,6 +152,13 @@ server <- function(input, output, session) {
                        "lwr", "upr"),]
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$rate_resetable_input <- renderUI({
+        times <- input$rate_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("rate_x", "Number of Events", value = NULL, min = 0),
+            numericInput("rate_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0))
+    })
 
     # meandiff ----
     output$meandiff_out <- renderPrint(meandiff_fn(input))
@@ -145,6 +171,15 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "ar"] <- "r"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$meandiff_resetable_input <- renderUI({
+        times <- input$meandiff_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("meandiff_n", "Number of observations in group 1",
+                         value = NULL),
+            "(the number of observations in group 2 is estimated based on group 1 and the allocation ratio)",
+            numericInput("meandiff_ciwidth", "Confidence interval width",
+                         value = NULL))
+    })
 
     # riskdiff ----
     output$riskdiff_out <- renderPrint(riskdiff_fn(input))
@@ -157,6 +192,17 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "ar"] <- "r"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$riskdiff_resetable_input <- renderUI({
+        times <- input$riskdiff_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("riskdiff_n1",
+                         "Number of observations in group 1",
+                         value = NULL),
+            "(the number of observations in group 2 is estimated based on group 1 and the allocation ratio)",
+            numericInput("riskdiff_ciwidth",
+                         "Confidence interval width",
+                         value = NULL))
+    })
 
     # or ----
     output$or_out <- renderPrint(or_fn(input))
@@ -168,6 +214,17 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "ar"] <- "r"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$or_resetable_input <- renderUI({
+        times <- input$or_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("or_n1",
+                         "Number of observations in group 1",
+                         value = NULL),
+            "(the number of observations in group 2 is estimated based on group 1 and the allocation ratio)",
+            numericInput("or_ciwidth",
+                         "Confidence interval width",
+                         value = NULL))
+    })
 
     # risk ratio ----
     output$riskratio_out <- renderPrint(riskratio_fn(input))
@@ -179,6 +236,17 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "ar"] <- "r"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$riskratio_resetable_input <- renderUI({
+        times <- input$riskratio_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("riskratio_n1",
+                         "Number of observations in group 1",
+                         value = NULL),
+            "(the number of observations in group 2 is estimated based on group 1 and the allocation ratio)",
+            numericInput("riskratio_ciwidth",
+                         "Confidence interval width",
+                         value = NULL))
+    })
 
     # rate ratio ----
     output$rateratio_out <- renderPrint(rateratio_fn(input))
@@ -190,6 +258,14 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "ar"] <- "r"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$rateratio_resetable_input <- renderUI({
+        times <- input$rateratio_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("rateratio_n_exp", "Number of observations in the exposed group", value = NULL, step = .1),
+            "(the number of observations in group 2 is estimated based on the number of exposed individuals and the allocation ratio)",
+            numericInput("rateratio_ciwidth", "Upper-lower ratio",
+                         value = NULL, step = .1))
+    })
 
     # correlation coefficient ----
     output$cor_out <- renderPrint(cor_fn(input))
@@ -201,6 +277,14 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "cr"] <- "r"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$cor_resetable_input <- renderUI({
+        times <- input$cor_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("cor_n", "Number of observations", value = NULL),
+            numericInput("cor_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, max = 1)
+            )
+    })
 
     # ICC ----
     output$icc_out <- renderPrint(icc_fn(input))
@@ -210,6 +294,14 @@ server <- function(input, output, session) {
         tmp1 <- res_vars[res_vars$column %in% names(tmp),]
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$icc_resetable_input <- renderUI({
+        times <- input$icc_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("icc_n", "Number of subjects", value = NULL),
+            numericInput("icc_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, max = 1)
+        )
+    })
 
     # limit of agreement ----
     output$limit_out <- renderPrint(limit_fn(input))
@@ -243,6 +335,15 @@ server <- function(input, output, session) {
         tmp1 <- res_vars[res_vars$column %in% names(tmp),]
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$limit_resetable_input <- renderUI({
+        times <- input$limit_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("limit_n", "Sample size",
+                         value = NULL),
+            numericInput("limit_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, max = 1)
+        )
+    })
 
     # kappa ----
     output$kappa_out <- renderPrint(kappa_fn(input))
@@ -252,6 +353,13 @@ server <- function(input, output, session) {
         tmp1 <- res_vars[res_vars$column %in% names(tmp),]
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$kappa_resetable_input <- renderUI({
+        times <- input$kappa_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("kappa_n", "Sample size", min = 0, value = NULL),
+            numericInput("kappa_ciwidth", "Confidence interval width", min = 0, value = NULL)
+        )
+    })
 
     # sens ----
     output$sens_out <- renderPrint(sens_fn(input))
@@ -263,6 +371,24 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "nsens"] <- "n"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$sens_resetable_input <- renderUI({
+        times <- input$sens_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("sens_ntot", "Total sample size",
+                         value = NULL),
+            uiOutput("sens_prev_ui"),
+            numericInput("sens_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, max = 1)
+        )
+    })
+    output$sens_prev_ui <- renderUI({
+        print(input$sens_ntot)
+        if(!is.na(input$sens_ntot)){
+            sliderInput("sens_prev", "Please also enter the prevalence",
+                        min = 0, max = 1, value = .4)
+        }
+    })
+
 
     # spec ----
     output$spec_out <- renderPrint(spec_fn(input))
@@ -274,6 +400,23 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "nspec"] <- "n"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$spec_resetable_input <- renderUI({
+        times <- input$spec_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("spec_ntot", "Total sample size",
+                         value = NULL),
+            uiOutput("spec_prev_ui"),
+            numericInput("spec_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, max = 1)
+        )
+    })
+    output$spec_prev_ui <- renderUI({
+        print(input$spec_ntot)
+        if(!is.na(input$spec_ntot)){
+            sliderInput("spec_prev", "Please also enter the prevalence",
+                        min = 0, max = 1, value = .4)
+        }
+    })
 
     # auc ----
     output$auc_out <- renderPrint(auc_fn(input))
@@ -301,6 +444,15 @@ server <- function(input, output, session) {
             ggplot2::annotate("text", x = .6, y = .25, label = "AUC") +
             ggplot2::annotate("text", x = .2, y = .85, label = "ROC")
     })
+    output$auc_resetable_input <- renderUI({
+        times <- input$auc_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("auc_n", "Total sample size",
+                         value = NULL),
+            numericInput("auc_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, max = 1)
+        )
+    })
 
     # likelihood ratio ----
     output$lrp_out <- renderPrint(lrp_fn(input))
@@ -315,6 +467,16 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "lr_p2"] <- "p2"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$lrp_resetable_input <- renderUI({
+        times <- input$lpr_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("lrp_n", "Total sample size",
+                         value = NULL),
+            "The number in each group (e.g. diseased and healthy) is calculated as sample size * prevalence (group 1, which might be diseased people) and sample size * 1-prevalence (group 2).",
+            numericInput("lrp_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, max = 1)
+        )
+    })
 
     output$lrn_out <- renderPrint(lrn_fn(input))
     output$lrn_code <- renderPrint(lrn_fn(input, TRUE))
@@ -328,6 +490,16 @@ server <- function(input, output, session) {
         tmp1$column[tmp1$column == "lr_p2"] <- "p2"
         tmp1[na.omit(match(names(tmp), tmp1$column)),]
     })
+    output$lrn_resetable_input <- renderUI({
+        times <- input$lpn_reset_input
+        div(id=letters[(times %% length(letters)) + 1],
+            numericInput("lrn_n", "Total sample size",
+                         value = NULL),
+            "The number in each group (e.g. diseased and healthy) is calculated as sample size * prevalence (group 1, which might be diseased people) and sample size * 1-prevalence (group 2).",
+            numericInput("lrn_ciwidth", "Confidence interval width",
+                         value = NULL, min = 0, max = 1)
+        )
+    })
 }
 
 # Run the application ----
